@@ -127,7 +127,7 @@ def seq(
             # If isoforms true, fetch sequences of isoforms instead
             if isoforms == True:
                 # Get ID type (gene, transcript, ...) using gget info
-                info_df = info(ensembl_ID, verbose=False)
+                info_df = info(ensembl_ID, verbose=False, ensembl_only=True)
 
                 # Check if Ensembl ID was found
                 if isinstance(info_df, type(None)):
@@ -229,7 +229,7 @@ def seq(
 
             for ensembl_ID in ens_ids_clean:
                 # Get ID type (gene, transcript, ...) using gget info
-                info_df = info(ensembl_ID, verbose=False)
+                info_df = info(ensembl_ID, verbose=False, ensembl_only=True)
 
                 # Check that Ensembl ID was found
                 if isinstance(info_df, type(None)):
@@ -292,7 +292,7 @@ def seq(
 
             for ensembl_ID in ens_ids_clean:
                 # Get ID type (gene, transcript, ...) using gget info
-                info_df = info(ensembl_ID, verbose=False)
+                info_df = info(ensembl_ID, verbose=False, ensembl_only=True)
 
                 # Check that Ensembl ID was found
                 if isinstance(info_df, type(None)):
