@@ -17,7 +17,7 @@ logging.basicConfig(
 # Mute numexpr threads info
 logging.getLogger("numexpr").setLevel(logging.WARNING)
 
-from .constants import ENSEMBL_FTP_URL, UNIPROT_IDMAPPING_API
+from .constants import ENSEMBL_FTP_URL, UNIPROT_IDMAPPING_API, ENS_TO_PDB_API
 
 
 def n_colors(nucleotide):
@@ -330,6 +330,28 @@ def get_uniprot_info(server, ensembl_id, verbose=True):
             descriptions.append(des_temp)
         df["uniprot_description"] = descriptions
 
+        # Get subcellular localisations for each result
+        subcel_locs_final = []
+        for i in np.arange(len(json["results"])):
+            subcel_locs = []
+            try:
+                for comment_idx in np.arange(len(json["results"][i]["comments"])):
+                    comment_json = json["results"][i]["comments"][comment_idx]
+                    if comment_json["commentType"] == "SUBCELLULAR LOCATION":
+                        for location_dict in comment_json["subcellularLocations"]:
+                            subcel_locs.append(location_dict["location"]["value"])
+            except:
+                pass
+            subcel_locs_final.append(subcel_locs)
+
+        if any(subcel_locs_final):
+            df["subcellular_localisation"] = subcel_locs_final
+        else:
+            # No subcellular localisation data will return as nan
+            nan_list = np.empty(len(subcel_locs_final))
+            nan_list[:] = np.NaN
+            df["subcellular_localisation"] = nan_list
+
         # Add query colunm
         df["query"] = ensembl_id
 
@@ -337,8 +359,8 @@ def get_uniprot_info(server, ensembl_id, verbose=True):
         if len(df) > 1:
             final_df = pd.DataFrame()
             for column in df.columns:
-                if column == "uni_synonyms":
-                    # Flatten synonym lists
+                if column == "uni_synonyms" or column == "subcellular_localisation":
+                    # Flatten synonym and subcellular_localisation lists
                     syn_lists = df[column].values
                     try:
                         flat_list = [item for sublist in syn_lists for item in sublist]
@@ -372,95 +394,121 @@ def get_uniprot_info(server, ensembl_id, verbose=True):
         return None
 
 
-def get_pdb_ids(uniprot_ids):
+# This function was replaced by the faster and more complete PDB API (see get_pdb_ids below)
+# def get_pdb_ids(uniprot_ids):
+#     """
+#     Function to fetch all PDB IDs linked to a list of UniProt IDs
+#     using UniProt's ID mapping (https://www.uniprot.org/help/id_mapping#example_uniparc).
+#     """
+
+#     retries = Retry(total=5, backoff_factor=0.25, status_forcelist=[500, 502, 503, 504])
+#     session = requests.Session()
+#     session.mount("https://", HTTPAdapter(max_retries=retries))
+
+#     # Sleep interval between
+#     POLLING_INTERVAL = 3
+
+#     def check_response(response):
+#         """
+#         Check response of HTTP request.
+#         """
+#         try:
+#             response.raise_for_status()
+#         except requests.HTTPError:
+#             logging.error(
+#                 f"UniProt ID mapping to fetch PDB IDs returned HTTP Error:\n{response.json()}"
+#             )
+#             return
+
+#     def post_mapping_request(uniprot_ids, from_id_type, to_id_type):
+#         """
+#         Post ID mapping request to UniProt ID mapping API.
+#         """
+#         # Post ID mapping request
+#         request = requests.post(
+#             f"{UNIPROT_IDMAPPING_API}/run",
+#             data={"from": from_id_type, "to": to_id_type, "ids": ",".join(uniprot_ids)},
+#         )
+
+#         check_response(request)
+
+#         # Return job ID
+#         return request.json()["jobId"]
+
+#     def check_id_mapping_results_ready(job_id):
+#         """
+#         Poll for status of mapping request.
+#         """
+#         while True:
+#             request = session.get(f"{UNIPROT_IDMAPPING_API}/status/{job_id}")
+#             check_response(request)
+#             j = request.json()
+#             if "jobStatus" in j:
+#                 # Sleep for POLLING_INTERVAL seconds if job status if "RUNNING"
+#                 if j["jobStatus"] == "RUNNING":
+#                     logging.info("Checking if PDB IDs are available...")
+#                     time.sleep(POLLING_INTERVAL)
+#                 else:
+#                     # Raise error if job status is other than "RUNNING"
+#                     raise Exception(request["jobStatus"])
+#             else:
+#                 # Return True if job is done running
+#                 return True
+
+#     def get_id_mapping_results_search(job_id):
+#         """
+#         Get mapping results from job ID.
+#         """
+#         # Get ID mapping results
+#         request = requests.get(
+#             f"{UNIPROT_IDMAPPING_API}/results/{job_id}",
+#         )
+
+#         check_response(request)
+
+#         # Return job ID
+#         return request.json()
+
+#     # Post mapping request
+#     job_id = post_mapping_request(
+#         uniprot_ids=uniprot_ids, from_id_type="UniProtKB_AC-ID", to_id_type="PDB"
+#     )
+
+#     # Get results once they are ready
+#     if check_id_mapping_results_ready(job_id):
+#         results = get_id_mapping_results_search(job_id)
+
+#     # Get list of PDB IDs
+#     pdb_ids = []
+#     if results["results"]:
+#         pdb_ids = results["results"][0].values()
+
+#     return list(pdb_ids)
+
+
+def get_pdb_ids(ens_id):
     """
-    Function to fetch all PDB IDs linked to a list of UniProt IDs
-    using UniProt's ID mapping (https://www.uniprot.org/help/id_mapping#example_uniparc).
+    Function to fetch all PDB IDs linked to an Ensembl ID.
+    using the PDBe API https://wwwdev.ebi.ac.uk/pdbe/aggregated-api/mappings/ensembl_to_pdb/[ens_id]
     """
 
-    retries = Retry(total=5, backoff_factor=0.25, status_forcelist=[500, 502, 503, 504])
-    session = requests.Session()
-    session.mount("https://", HTTPAdapter(max_retries=retries))
+    res = requests.get(ENS_TO_PDB_API + ens_id)
 
-    # Sleep interval between
-    POLLING_INTERVAL = 3
+    if not res.ok:
+        # If no PDB IDs were found, return None
+        return None
 
-    def check_response(response):
-        """
-        Check response of HTTP request.
-        """
-        try:
-            response.raise_for_status()
-        except requests.HTTPError:
-            logging.error(
-                f"UniProt ID mapping to fetch PDB IDs returned HTTP Error:\n{response.json()}"
-            )
-            return
+    try:
+        pdb_dict = res.json()[ens_id]["mappings"]
+    # If no PDB IDs were found, return None
+    except KeyError:
+        return None
 
-    def post_mapping_request(uniprot_ids, from_id_type, to_id_type):
-        """
-        Post ID mapping request to UniProt ID mapping API.
-        """
-        # Post ID mapping request
-        request = requests.post(
-            f"{UNIPROT_IDMAPPING_API}/run",
-            data={"from": from_id_type, "to": to_id_type, "ids": ",".join(uniprot_ids)},
-        )
-
-        check_response(request)
-
-        # Return job ID
-        return request.json()["jobId"]
-
-    def check_id_mapping_results_ready(job_id):
-        """
-        Poll for status of mapping request.
-        """
-        while True:
-            request = session.get(f"{UNIPROT_IDMAPPING_API}/status/{job_id}")
-            check_response(request)
-            j = request.json()
-            if "jobStatus" in j:
-                # Sleep for POLLING_INTERVAL seconds if job status if "RUNNING"
-                if j["jobStatus"] == "RUNNING":
-                    logging.info("Checking if PDB IDs are available...")
-                    time.sleep(POLLING_INTERVAL)
-                else:
-                    # Raise error if job status is other than "RUNNING"
-                    raise Exception(request["jobStatus"])
-            else:
-                # Return True if job is done running
-                return True
-
-    def get_id_mapping_results_search(job_id):
-        """
-        Get mapping results from job ID.
-        """
-        # Get ID mapping results
-        request = requests.get(
-            f"{UNIPROT_IDMAPPING_API}/results/{job_id}",
-        )
-
-        check_response(request)
-
-        # Return job ID
-        return request.json()
-
-    # Post mapping request
-    job_id = post_mapping_request(
-        uniprot_ids=uniprot_ids, from_id_type="UniProtKB_AC-ID", to_id_type="PDB"
-    )
-
-    # Get results once they are ready
-    if check_id_mapping_results_ready(job_id):
-        results = get_id_mapping_results_search(job_id)
-
-    # Get list of PDB IDs
     pdb_ids = []
-    if results["results"]:
-        pdb_ids = results["results"][0].values()
+    for entry in pdb_dict:
+        pdb_ids.append(entry["pdb_id"])
 
-    return list(pdb_ids)
+    return list(set(pdb_ids))
 
 
 def wrap_cols_func(df, cols):
